import serial
import threading
import socket
import json
import time
from http.server import SimpleHTTPRequestHandler, HTTPServer
from forum import Forum
import base4096

SERIAL_PORT = "COM3"  # Adjust for your setup
BAUD_RATE = 115200
MESH_PORT = 9000

forum = Forum()
stop_flag = False

def serial_to_forum():
    global stop_flag
    while not stop_flag:
        try:
            with serial.Serial(SERIAL_PORT, BAUD_RATE, timeout=1) as ser:
                print(f"Connected to serial port {SERIAL_PORT}")
                while not stop_flag:
                    line = ser.readline().decode(errors='ignore').strip()
                    if line:
                        try:
                            data = base4096.decode(line)
                            forum.receive_post(data)
                        except Exception as e:
                            print(f"Decode error: {e}")
        except serial.SerialException:
            print(f"Serial port {SERIAL_PORT} unavailable. Retrying in 3s...")
            time.sleep(3)

def mesh_peer_discovery():
    global stop_flag
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
    sock.bind(("", MESH_PORT))
    print(f"Mesh peer discovery running on UDP port {MESH_PORT}")
    while not stop_flag:
        try:
            data, addr = sock.recvfrom(1024)
            msg = data.decode()
            print(f"Received mesh hello from {addr}: {msg}")
        except Exception as e:
            print(f"Discovery error: {e}")

def serve_webui():
    import os
    os.chdir("webui")
    server = HTTPServer(("localhost", 8787), SimpleHTTPRequestHandler)
    print("Serving forum UI at http://localhost:8787")
    server.serve_forever()

if __name__ == "__main__":
    threads = []
    t_serial = threading.Thread(target=serial_to_forum, daemon=True)
    t_mesh = threading.Thread(target=mesh_peer_discovery, daemon=True)
    threads.extend([t_serial, t_mesh])
    for t in threads:
        t.start()
    serve_webui()
    stop_flag = True
    for t in threads:
        t.join()
