from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives import serialization, hashes
from cryptography.exceptions import InvalidSignature

def generate_keys():
    private_key = ec.generate_private_key(ec.SECP256K1())
    public_key = private_key.public_key()

    with open("private.pem", "wb") as f:
        f.write(private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.NoEncryption()))

    with open("public.pem", "wb") as f:
        f.write(public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo))

def sign_data(data):
    with open("private.pem", "rb") as f:
        private_key = serialization.load_pem_private_key(f.read(), password=None)
    signature = private_key.sign(data.encode(), ec.ECDSA(hashes.SHA256()))
    return signature.hex()

def verify_signature(data, signature_hex, pub_key_pem):
    from cryptography.hazmat.primitives.serialization import load_pem_public_key
    signature = bytes.fromhex(signature_hex)
    public_key = load_pem_public_key(pub_key_pem.encode())
    try:
        public_key.verify(signature, data.encode(), ec.ECDSA(hashes.SHA256()))
        return True
    except InvalidSignature:
        return False
